/**
 * editor_plugin_src.js
 */

(function() {
   // Load plugin specific language pack
   tinymce.PluginManager.requireLangPack('mm_media');

   // Create plugin
   tinymce.create('tinymce.plugins.MM_MediaPlugin', {
		    /**
		     * Returns information about the plugin as a name/value array.
		     * The current keys are longname, author, authorurl, infourl and version.
		     *
		     * @return {Object} Name/value array containing information about the plugin.
		     */
		    getInfo : function() {
		      return {
		        longname : 'Monster Menus Media plugin',
		        author : 'Amherst College',
		        authorurl : 'https://www.amherst.edu',
		        infourl : 'https://www.amherst.edu/offices/it/web/monstermenus/mm_media',
		        version : "1.5"
		      };
		    },
		    /**
		     * Initializes the plugin, this will be executed after the plugin has been created.
		     * This call is done before the editor instance has finished it's initialization so use the onInit event
		     * of the editor instance to intercept that event.
		     *
		     * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
		     * @param {string} url Absolute URL to where the plugin is located.
		     */
		    init : function(ed, url) {
		      // Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand('mm_media');
		      ed.addCommand('mm_media_assist', function() {
				      var base_url = url.substring(0, url.indexOf('sites/all/modules'))+'media_assist/load?location='+document.location.pathname;
				      ed.windowManager.open({
					                      file : base_url,
					                      width : 900,
					                      height : 450,
					                      scrollbars: true,
					                      resizable: true,
					                      inline : 1
				                            }, {
					                      plugin_url : url, // Plugin absolute URL
					                      some_custom_arg : 'custom arg' // Custom argument
				                            });
			            });
		      // Register mm_media button
		      ed.addButton('mm_media', {
				     title : 'Insert file upload',
				     cmd : 'mm_media_assist',
				     image : url + '/img/mm_media.gif'
			           });
                      
		      // Add a node change handler, selects the button in the UI when a image is selected
		      ed.onNodeChange.add(function(ed, cm, n) {
				            cm.setActive('mm_media', n.nodeName == 'IMG');
			                  });
		    },

		    /**
		     * Creates control instances based in the incoming name. This method is normally not
		     * needed since the addButton method of the tinymce.Editor class is a more easy way of adding buttons
		     * but you sometimes need to create more complex controls like listboxes, split buttons etc then this
		     * method can be used to create those.
		     *
		     * @param {String} n Name of the control to create.
		     * @param {tinymce.ControlManager} cm Control manager to use inorder to create new control.
		     * @return {tinymce.ui.Control} New control instance or null if no control was created.
		     */
		    createControl : function(n, cm) {
		      return null;
		    }
	          });

   // Register plugin
   tinymce.PluginManager.add('mm_media', tinymce.plugins.MM_MediaPlugin);
})();