<?php

/**
 * @file
 * Administrative functions
 */

/**
 * Called from monster_menus_menu().
 */
function mm_admin_menu() {
  $items = array();
  $base = array('file' => 'mm_admin.inc');

  $items['admin/settings/mm'] = $base + array(
    'title' => 'Monster Menus',
    'description' => "Change various settings that affect Monster Menus' appearance and behavior",
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_config'),
    'access callback' => '_mm_menu_access_any_admin');

  // Administer->Monster Menus
  $items['admin/mm'] = array(
    'title' => 'Monster Menus',
    'description' => 'Perform various administrative tasks in Monster Menus.',
    'page callback' => 'system_admin_menu_block_page',
    'file' => 'system.admin.inc',
    'file path' => drupal_get_path('module', 'system'),
    'access callback' => '_mm_menu_access_any_admin');
  $items['admin/mm/settings'] = $base + array(
    'title' => 'Settings',
    'description' => "Change various settings that affect Monster Menus' appearance and behavior",
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_config'),
    'access callback' => '_mm_menu_access_any_admin');
  $items['admin/mm/browse'] = $base + array(
    'title' => 'Browse tree',
    'description' => 'View sections of the Monster Menus tree',
    'page callback' => 'mm_admin_browse',
    'type' => MENU_NORMAL_ITEM,
    'access callback' => '_mm_menu_access_any_admin');
  $items['admin/mm/dump'] = $base + array(
    'title' => 'Export tree',
    'description' => 'Export the entire Monster Menus tree as a CSV file',
    'page callback' => 'mm_admin_dump',
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/mm/vgroups'] = $base + array(
    'title' => 'Redo virtual groups',
    'description' => 'Force the membership of all virtual groups to be regenerated',
    'page callback' => 'mm_regenerate_vgroup',
    'access callback' => 'user_access',
    'access arguments' => array('administer all groups'));
  $items['admin/mm/integrity'] = $base + array(
    'title' => 'Verify integrity of database relationships',
    'description' => "Check the relationships between Monster Menus' tables for missing records",
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_verify_integrity'),
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/mm/sort'] = $base + array(
    'title' => 'Verify sort index',
    'description' => 'Check for problems in the way menu entries are sorted and optionally correct them',
    'page callback' => 'mm_admin_validate_sort_index',
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/mm/reassign'] = $base + array(
    'title' => 'Reassign user content',
    'description' => 'Give ownership of all pages and content owned by a particular user to someone else',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_reassign_content'),
    'access callback' => '_mm_menu_access_all_admin');

  // Administer->Monster Menus->Manage sites
  $items['admin/mm/sites'] = $base + array(
    'title' => 'Manage sites',
    'description' => 'Set a different homepage for each Drupal site',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_list_sites'),
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/mm/sites/list'] = $base + array(
    'title' => 'List',
    'weight' => -10,
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/mm/sites/add'] = $base + array(
    'title' => 'Add',
    'page callback' => 'mm_admin_edit_site',
    'type' => MENU_LOCAL_TASK,
    'access callback' => '_mm_menu_access_all_admin');

  // Administer->Blocks
  $items['admin/build/block/mm'] = $base + array(
    'title' => 'MM blocks',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_list_blocks'),
    'type' => MENU_LOCAL_TASK,
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/build/block/mm/list'] = $base + array(
    'title' => 'List',
    'weight' => -10,
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/build/block/mm/edit'] = $base + array(
    'title' => 'Configure block',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_edit_block'),
    'type' => MENU_CALLBACK,
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/build/block/mm/add'] = $base + array(
    'title' => 'Add block',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_edit_block'),
    'type' => MENU_LOCAL_TASK,
    'access callback' => '_mm_menu_access_all_admin');
  $items['admin/build/block/mm/delete'] = $base + array(
    'title' => 'Delete block',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_delete_block'),
    'type' => MENU_CALLBACK,
    'access callback' => '_mm_menu_access_all_admin');

  // Administer->User Management->Users
  $items['admin/user/user/by-uid'] = $base + array(
    'title' => 'Find by name',
    'type' => MENU_LOCAL_TASK,
    'page callback' => 'drupal_get_form',
    'page arguments' => array('mm_admin_find_user'),
    'access arguments' => array('administer users'));

  return $items;
}

function mm_admin_config() {
  $form = array();

  if (user_access('administer all menus')) {
    $form['general'] = array(
      '#type' => 'fieldset',
      '#title' => t('General'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $form['general']['mm_recycle_auto_empty'] = array(
      '#type' => 'select',
      '#title' => t('Automatic recycle bin deletion interval'),
      '#description' => t('Automatically delete content in recycle bins that has been there longer than this amount of time'),
      '#default_value' => variable_get('mm_recycle_auto_empty', 0),
      '#options' => array(
          -1                       => t('(don\'t use recycle bins)'),
          0                        => t('(never auto delete)'),
          30*60                    => t('30 minutes'),
          60*60                    => t('1 hour'),
          2*60*60                  => t('2 hours'),
          6*60*60                  => t('6 hours'),
          12*60*60                 => t('12 hours'),
          24*60*60                 => t('1 day'),
          2*24*60*60               => t('2 days'),
          3*24*60*60               => t('3 days'),
          7*24*60*60               => t('1 week'),
          2*7*24*60*60             => t('2 weeks'),
          30*24*60*60              => t('30 days'),
          60*24*60*60              => t('60 days'),
          intval(365/4*24*60*60)   => t('3 months'),
          intval(365/2*24*60*60)   => t('6 months'),
          intval(365/4*3*24*60*60) => t('9 months'),
          365*24*60*60             => t('1 year'),
      )
    );
    $form['general']['mm_access_cache_time'] = array(
      '#type' => 'select',
      '#options' => array(0 => t('(disabled)'), 30 => t('30 seconds'), 60 => t('1 minute'), 120 => t('2 minutes'), 180 => t('3 minutes'), 240 => t('4 minutes'), 300 => t('5 minutes'), 600 => t('10 minutes'), 900 => t('15 minutes'), 1200 => t('20 minutes'), 1800 => t('30 minutes'), 2700 => t('45 minutes'), 3600 => t('1 hour')),
      '#title' => t('Permissions cache time'),
      '#description' => t("Save time by caching the data needed to determine if a given user has access to a page or piece of content. The cache is automatically cleared whenever a piece content or any of its parent pages' permissions are modified. Setting this value too high can lead to lots of data being stored in the cache table."),
      '#default_value' => variable_get('mm_access_cache_time', 0),
    );

    $form['vgroup'] = array(
      '#type' => 'fieldset',
      '#title' => t('Virtual Groups'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $form['vgroup']['mm_vgroup_regen_chunk'] = array(
      '#type' => 'textfield',
      '#title' => t('Chunk size used to split virtual group regeneration queries'),
      '#size' => 10, '#default_value' => variable_get('mm_vgroup_regen_chunk', 15),
      '#description' => t('To gain speed when regenerating dirty virtual groups, the separate queries are concatenated into one big query with a UNION. If the maximum SQL query buffer length is being exceeded or the large queries are taking too much memory, this value should be reduced.'),
    );
    $form['vgroup']['mm_vgroup_regen_chunks_per_run'] = array(
      '#type' => 'textfield',
      '#title' => t('Number of concatenated virtual group queries per cron run'),
      '#size' => 10, '#default_value' => variable_get('mm_vgroup_regen_chunks_per_run', 50),
      '#description' => t('If cron is taking too long to complete, this value should be reduced. The total number of dirty virtual groups updated per cron run is this number times the chunk size.'),
    );
    $form['vgroup']['mm_vgroup_errors_email'] = array(
      '#type' => 'textfield',
      '#title' => t('Where to send virtual group warnings'),
      '#default_value' => variable_get('mm_vgroup_errors_email', variable_get('site_mail', ini_get('sendmail_from'))),
      '#description' => t('Warnings are generated for any virtual groups that decrease in size too rapidly. This is the e-mail address where warnings are sent. If left blank, the site e-mail address is used.'),
      '#maxlength' => 1024,
      '#size' => 100,
    );

    $form['mm_page'] = array(
      '#type' => 'fieldset',
      '#title' => t('Page Display'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $form['mm_page']['mm_hide_empty_pages'] = array(
      '#type' => 'checkbox',
      '#title' => t('Hide empty pages in menus'),
      '#default_value' => variable_get('mm_hide_empty_pages', FALSE),
    );
    $form['mm_page']['mm_enable_rss'] = array(
      '#type' => 'checkbox',
      '#title' => t('Allow content creators to control the availability of RSS feeds on a per-page basis'),
      '#default_value' => variable_get('mm_enable_rss', FALSE),
    );

    $form['mm_node'] = array(
      '#type' => 'fieldset',
      '#title' => t('Node Display'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $form['mm_node']['mm_node_alert_frequency'] = array(
      '#type' => 'select',
      '#title' => t('Frequency of system alerts'),
      '#default_value' => variable_get('mm_node_alert_frequency', 'once'),
      '#options' => array(
        'once' => t('Show only once'),
        'login' => t('Show at every login'),
        'constantly' => t('Show at every page load')
      ),
    );
    $form['mm_node']['comments'] = array(
      '#type' => 'fieldset',
      '#title' => t('Comments'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $form['mm_node']['comments']['mm_show_count_instead_of_comments'] = array(
      '#type' => 'checkbox',
      '#title' => t('Show comment count instead of full comments'),
      '#description' => t('This option takes effect when viewing pages. When checked, nodes having comments show a link with the number of comments. Clicking on the link displays the node by itself, with the comments. If unchecked, all comments are displayed under their nodes, on the same page.'),
      '#default_value' => variable_get('mm_show_count_instead_of_comments', FALSE),
    );
    $form['mm_node']['comments']['mm_finegrain_comment_readability'] = array(
      '#type' => 'checkbox',
      '#title' => t('Control comment readability at the node level'),
      '#description' => t('This option lets users say who can read comments posted to each node on an individual basis. A default value for new nodes can also be set at the page level.'),
      '#default_value' => variable_get('mm_finegrain_comment_readability', FALSE),
    );
    $labels = variable_get('mm_comments_readable_labels', array());
    $labels[] = array();
    if (count($labels) == 1) $labels[] = array();
    $form['mm_node']['comments']['mm_comments_readable_labels'] = array(
      '#type' => 'markup',
      '#prefix' => '<table><tr><th>' . t('Permission') . '</th><th>' . t('Description') . '</th></tr>',
      '#suffix' => '</table>',
      '#description' => 'foo',
      '#tree' => TRUE
    );
    $i = 0;
    foreach ($labels as $label) {
      $form['mm_node']['comments']['mm_comments_readable_labels'][$i]['perm'] = array(
        '#type' => 'textfield',
        '#prefix' => '<tr><td>',
        '#default_value' => isset($label['perm']) ? $label['perm'] : '',
        '#size' => 30,
        '#suffix' => '</td><td>',
      );
      $form['mm_node']['comments']['mm_comments_readable_labels'][$i]['desc'] = array(
        '#type' => 'textfield',
        '#suffix' => '</td></tr>',
        '#default_value' => isset($label['desc']) ? $label['desc'] : '',
        '#size' => 30,
      );
      $i++;
    }
    $form['mm_node']['comments']['desc'] = array(
      '#type' => 'item',
      '#description' => t('<p><em>Permission</em> is the label appearing on the !url page; these are effectively ANDed with the <em>access comments</em> permission. Example: <em>comments readable by everyone</em></p><p><em>Description</em> is what users see in the list of choices for setting readability at the page/node level, it answers the question, "Who can read comments?" Example: <em>everyone</em></p><p>To remove a row, clear either value. <strong>Changing data in the Permission column or removing rows may affect the readability of comments in existing nodes!</strong> Don\'t forget to update the permissions after making changes here.</p>', array('!url' => l(t('Permissions'), 'admin/user/permissions', array('fragment' => 'module-monster_menus'))))
    );

    $form['mm_userhome'] = array(
      '#type' => 'fieldset',
      '#title' => t('User Home Pages'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $form['mm_userhome']['mm_use_user_dir'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use user home directories'),
      '#description' => t('When enabled, each newly-added user gets a personal home page, starting at !url. Note: If you disable and then re-enable this option, any users created during the time it was disabled will not have home pages.', array('!url' => l('users', 'users'))),
      '#default_value' => variable_get('mm_use_user_dir', TRUE),
    );
    $form['mm_userhome']['mm_use_virtual_user_dir'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use virtual user directories'),
      '#description' => t('If you have many users, the entire user list at !url can get very long. This feature will split the users into smaller chunks, based on the letter of the alphabet with which their name begins.', array('!url' => l('users', 'users'))),
      '#default_value' => variable_get('mm_use_virtual_user_dir', TRUE),
    );
    $form['mm_userhome']['mm_default_homepage'] = array(
      '#type' => 'textarea',
      '#title' => t('Default personal homepage message'),
      '#description' => t('What users see when viewing their own, empty homepage. Provide some instructions telling them how to create content.'),
      '#default_value' => variable_get('mm_default_homepage', ''),
    );

    $form['mm_username'] = array(
      '#type' => 'fieldset',
      '#title' => t('User Names'),
      '#description' => t('These names are displayed in content attribution lines and group membership lists.'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $form['mm_username']['mm_username_anon'] = array(
      '#type' => 'textfield',
      '#title' => t('Long name of the Anonymous user'),
      '#default_value' => variable_get('mm_username_anon', t('Anonymous user')),
    );
    $form['mm_username']['mm_username_admin'] = array(
      '#type' => 'textfield',
      '#title' => t('Long name of the Administrator user'),
      '#default_value' => variable_get('mm_username_admin', t('Administrator')),
    );
    $form['mm_username']['mm_username_disabled'] = array(
      '#type' => 'textfield',
      '#title' => t('Long name for all disabled users'),
      '#default_value' => variable_get('mm_username_disabled', t('Inactive user')),
    );

    $form['mm_nodelist'] = array(
      '#type' => 'fieldset',
      '#title' => t('Node Chooser'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['mm_nodelist']['mm_nodelist_pager_limit'] = array(
      '#type' => 'select',
      '#title' => t('Number of nodes to show per page in the node chooser'),
      '#options' => array(
        10 => 10,
        20 => 20,
        50 => 50,
        100 => 100
      ),
      '#default_value' => variable_get('mm_nodelist_pager_limit', 20),
    );

    $form['mm_sitemap'] = array(
      '#type' => 'fieldset',
      '#title' => t('Site Map'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['mm_sitemap']['help'] = array(
      '#type' => 'markup',
      '#value'=> t('<p>Monster Menus will respond to a request for <code>/-mm-sitemap</code> by generating a standard <code>/sitemap.xml</code> file. You should call this URL periodically, in the same way you do cron.php, but less frequently. Once it has been generated, the <code>sitemap.xml</code> file contains links to any pages that are publicly readable, and not hidden or recycled.</p>'),
    );
    $form['mm_sitemap']['mm_sitemap_exclude_list'] = array(
      '#type' => 'textarea',
      '#wysiwyg' => FALSE,
      '#title' => t('Paths to exclude from the sitemap'),
      '#default_value' => join("\n", variable_get('mm_sitemap_exclude_list', array())),
      '#description' => t('A list of paths, one per line, which should not be part of the <code>sitemap.xml</code>. Do not include leading or trailing slashes. Example: <code>foo/bar/baz</code>'),
    );
    $form['mm_sitemap']['mm_sitemap_max_level'] = array(
      '#type' => 'select',
      '#options' => array(-1 => t('(disabled)'), 0 => t('(Home only)'), 1 => t('1 level'), 2 => 2, 3 => 3, 4 => 4, 5 => 5, 6 => 6, 7 => 7, 8 => 8, 9 => 9, 10 => 10, 11 => 11, 12 => 12, 13 => 13, 14 => 14, 15 => 15, 1000 => t('(unlimited)')),
      '#title' => t('Number of levels to generate'),
      '#default_value' => variable_get('mm_sitemap_max_level', MM_SITEMAP_MAX_LEVEL_DEFAULT),
      '#description' => t('The maximum depth in the tree to use for the sitemap. Set this too high and, on a large site, your sitemap.xml file may become too large to be useful.'),
    );

    $form['#submit'][] = 'mm_admin_config_submit';
  }

  mm_module_invoke_all_array('mm_config_alter', array(&$form));

  if ($form) return system_settings_form($form);

  $form['msg'] = array('#value' => t('You are not allowed to change any of the settings.'));
  return $form;
}

function mm_admin_config_validate($form, &$form_state) {
  foreach ($form_state['values']['mm_comments_readable_labels'] as $index => $row)
    if (empty($row['perm']) || empty($row['desc']))
      unset($form_state['values']['mm_comments_readable_labels'][$index]);

  foreach (preg_split('/\s*,\s*/', $form_state['values']['mm_vgroup_errors_email']) as $email) {
    if ($error = user_validate_mail($email)) {
      form_set_error('mm_vgroup_errors_email', $error);
    }
  }

  $form_state['values']['mm_sitemap_exclude_list'] = preg_split('{/*\s*[\r\n]+\s*/*}', trim($form_state['values']['mm_sitemap_exclude_list'], " \r\n/"), -1, PREG_SPLIT_NO_EMPTY);
}

function mm_admin_config_submit($form, &$form_state) {
  // If mm_finegrain_comment_readability has never been set before, create the
  // default settings using the current 'access comments' setting.
  if ($form_state['values']['mm_finegrain_comment_readability'] && !variable_get('mm_finegrain_comment_readability_ever_set', FALSE)) {
    variable_set('mm_finegrain_comment_readability_ever_set', TRUE);
    db_query("UPDATE {permission} SET perm = CONCAT(perm, ', %s') WHERE perm REGEXP '(^|, )access comments(,|\$)'", MM_COMMENT_READABILITY_DEFAULT);
    db_query("UPDATE {permission} SET perm = CONCAT(perm, ', access comments') WHERE perm NOT REGEXP '(^|, )access comments(,|\$)'");
    drupal_set_message(t('Because you enabled the <em>Control comment readability at the node level</em> setting for the first time, the <em>access comments</em> permission has been enabled for all roles. This is necessary in order for this feature to work.'));
    drupal_set_message(t('You should now go to !url to set the roles for each permission you just created.', array('!url' => l(t('Permissions'), 'admin/user/permissions', array('fragment' => 'module-monster_menus')))));
  }

  // If changing to/from disabled sitemap, we need to rebuild the menu.
  if ($form_state['values']['mm_sitemap_max_level'] != $form['mm_sitemap']['mm_sitemap_max_level']['#default_value'] && ($form_state['values']['mm_sitemap_max_level'] < 0 || $form['mm_sitemap']['mm_sitemap_max_level']['#default_value'] < 0)) {
    variable_set('mm_sitemap_max_level', $form_state['values']['mm_sitemap_max_level']);
    menu_rebuild();
  }
}

/**
 * Verify database relationships
 */
function mm_admin_verify_integrity() {
  foreach (module_list() as $module)
    module_load_include('install', $module);

  $tests = module_invoke_all('mm_verify_integrity');
  if (!$tests) return t('There are no tests to perform.');

  $form = array();
  $i = 0;
  foreach ($tests as $heading => $list) {
    $fieldset = "fs$i";
    $form[$fieldset] = array(
      '#type' => 'fieldset',
      '#title' => $heading,
      '#collapsible' => TRUE,
    );
    $form[$fieldset]['header'] = array(
      '#type' => 'value',
      '#value' => array(
        array('data' => t('Test')),
        array('data' => t('Result')),
      ),
    );

    foreach ($list as $description => $test)
      $form[$fieldset][] = array(
        'test' => array('#value' => check_plain(db_prefix_tables($test))),
        'description' => array('#value' => check_plain($description)),
        'result' => array('#value' => db_result(db_query("SELECT COUNT(*) FROM $test"))),
      );

    $i++;
  }
  return $form;
}

function mm_admin_list_sites() {
  $form['header'] = array(
    '#type' => 'value',
    '#value' => array(
      array('data' => t('Name'), 'field' => 'name'),
      array('data' => t('Current Homepage')),
      array('data' => t('Operations'), 'colspan' => '2'),
    ),
  );
  $result = pager_query("SELECT mmtid, name FROM {mm_tree} WHERE parent = 1 AND name NOT LIKE '.%'" . tablesort_sql($form['header']['#value']), 20, 0, NULL);

  $destination = drupal_get_destination();
  $page_mmtids = array();
  while ($site = db_fetch_object($result)) {
    $form['name'][$site->mmtid] = array('#value' => l(mm_content_expand_name($site->name), "mm/$site->mmtid"));
    $form['operations'][$site->mmtid][] = array('#value' => l(t('edit'), 'mm/'. $site->mmtid . '/settings', array('query' => $destination)));
    if ($site->mmtid != mm_home_mmtid()) {
      $form['operations'][$site->mmtid][] = array('#value' => l(t('delete'), 'mm/'. $site->mmtid . '/settings/delete', array('query' => $destination)));
    }
    $page_mmtids[$site->mmtid] = '';
  }
  $form['current'] = array(
    '#type' => 'radios',
    '#options' => $page_mmtids,
    '#default_value' => mm_home_mmtid(),
  );
  $form['pager'] = array('#value' => theme('pager', NULL, 20, 0));
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Set the current homepage'),
  );

  return $form;
}

function mm_admin_list_sites_submit($form, &$form_state) {
  $conf_path = conf_path();
  $list = variable_get('mm_home_mmtid', MM_HOME_MMTID_DEFAULT);
  if (!is_array($list)) {
    $list = array($conf_path => $list);
  }
  $list[$conf_path] = $form_state['values']['current'];
  variable_set('mm_home_mmtid', $list);
  drupal_set_message(t('The current site homepage has been changed.'));
}

function mm_admin_edit_site() {
  module_load_include('inc', 'monster_menus', 'mm_ui_content');
  return mm_ui_content(1, 'sub');
}

function mm_admin_list_blocks() {
  $form['header'] = array(
    '#type' => 'value',
    '#value' => array(
      array('data' => t('Name'), 'field' => 'name'),
      array('data' => t('Title'), 'field' => 'title'),
      array('data' => t('Flags')),
      array('data' => t('Operations'), 'colspan' => '2'),
    ),
  );
  $result = pager_query('SELECT * FROM {mm_block}' . tablesort_sql($form['header']['#value']), 20, 0, NULL);

  $destination = drupal_get_destination();
  while ($block = db_fetch_object($result)) {
    $form['name'][$block->bid] = array('#value' => check_plain($block->name));
    $form['title'][$block->bid] = array('#value' => check_plain($block->title));
    $flags = array();
    $fields = array(
      'title_is_cat' => t('Use page as title'),
      'allow_rss' => t('RSS feed'),
      'show_node_contents' => t('Show node contents'),
      'admin_only' => t('Admin. only')
    );
    foreach ($fields as $field => $label) {
      if ($block->$field) $flags[] = $label;
    }
    $form['flags'][$block->bid] = array('#value' => join(', ', $flags));
    $form['operations'][$block->bid][] = array('#value' => l(t('edit'), 'admin/build/block/mm/edit/'. $block->bid, array('query' => $destination)));
    $form['operations'][$block->bid][] = array('#value' => l(t('delete'), 'admin/build/block/mm/delete/'. $block->bid, array('query' => $destination)));
  }
  $form['pager'] = array('#value' => theme('pager', NULL, 20, 0));

  return $form;
}

function mm_admin_edit_block(&$form_state, $bid = NULL) {
  $blocks = mm_content_get_blocks();
  if (isset($bid) && isset($blocks[$bid])) {
    $form['bid'] = array('#type' => 'value', '#value' => $bid);
    $block = $blocks[$bid];
  }
  else {
    $block = array(
      'info' => '',
      'title' => '',
      'help' => '',
      'title_is_cat' => FALSE,
      'allow_rss' => FALSE,
      'show_node_contents' => FALSE,
      'admin_only' => FALSE,
    );
  }

  $x = mm_ui_strings(FALSE);
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#default_value' => $block['info'],
    '#size' => 40, '#maxlength' => 128,
    '#required' => TRUE,
    '#description' => t('The name visible to users in the Appearance settings for a @thing', $x)
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $block['title'],
    '#size' => 40, '#maxlength' => 256,
    '#description' => t('An optional title to be placed above the block')
  );
  $form['help'] = array(
    '#type' => 'textarea',
    '#title' => t('Help text'),
    '#default_value' => $block['help'],
    '#rows' => 4,
    '#description' => t('The text which appears in the tooltip, describing when to use this block')
  );
  $form['title_is_cat'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use @thing as title', $x),
    '#default_value' => $block['title_is_cat'],
    '#description' => t('If checked, the title of the block\'s @thing is used as the block title instead of the Title field, above.', $x)
  );
  $form['allow_rss'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow RSS feed'),
    '#default_value' => $block['allow_rss'],
    '#description' => t('If checked, automatically generate an additional RSS feed <link> tag for this block.')
  );
  $form['show_node_contents'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show node contents'),
    '#default_value' => $block['show_node_contents'],
    '#description' => t('If checked, show the contents of all nodes in this @thing instead of links.', $x)
  );
  $form['admin_only'] = array(
    '#type' => 'checkbox',
    '#title' => t('Admin. only'),
    '#default_value' => $block['admin_only'],
    '#description' => t('If checked, only users with the "administer all menus" permission can assign @things to this block.', $x)
  );
  $form['submit'] = array('#type' => 'submit',
      '#value' => isset($block) ? t('Save settings') : t('Add block'));
  return $form;
}

function mm_admin_edit_block_submit($form, &$form_state) {
  $form_vals =& $form_state['values'];
  if (isset($form_vals['bid'])) {
    db_query("UPDATE {mm_block} SET name = '%s', title = '%s', help = '%s', title_is_cat = %d, allow_rss = %d, show_node_contents = %d, admin_only = %d WHERE bid = %d",
        $form_vals['name'], $form_vals['title'], $form_vals['help'], $form_vals['title_is_cat'], $form_vals['allow_rss'], $form_vals['show_node_contents'], $form_vals['admin_only'], $form_vals['bid']);
    if (drupal_get_destination()) return;
  }
  else {
    db_query("INSERT INTO {mm_block} (name, title, help, title_is_cat, allow_rss, show_node_contents, admin_only) VALUES('%s', '%s', '%s', %d, %d, %d, %d)",
        $form_vals['name'], $form_vals['title'], $form_vals['help'], $form_vals['title_is_cat'], $form_vals['allow_rss'], $form_vals['show_node_contents'], $form_vals['admin_only']);
    drupal_set_message(t('The new block has been created.'));
  }
  $form_state['redirect'] = 'admin/build/block/mm';
}

function mm_admin_delete_block(&$form_state, $bid = NULL) {
  $blocks = mm_content_get_blocks();
  if (isset($bid) && isset($blocks[$bid])) {
    $form['bid'] = array('#type' => 'value', '#value' => $bid);
    $block = $blocks[$bid];

    $dest = drupal_get_destination();
    if (!$dest) $dest = 'admin/build/block/mm';
    else $dest = urldecode(str_replace('destination=', '', $dest));

    $msg = t('Are you sure you want to delete the block %name?', array('%name' => $block['info']));
    $count = db_result(db_query('SELECT COUNT(*) FROM {mm_tree_block} WHERE bid = %d', $bid));
    if ($count) {
      $msg .= format_plural($count, ' <font color=red>There is a @thing using this block.</font>', ' <font color=red>There are @count @things using this block.</font>', mm_ui_strings(FALSE));
    }

    module_load_include('inc', 'monster_menus', 'mm_ui_content_delete');
    _mm_ui_content_delete_confirm_form($form, $msg, t('Delete'), $dest);
    return $form;
  }
  else {
    drupal_set_message(t('Block not found'));
  }
}

function mm_admin_delete_block_submit($form, &$form_state) {
  db_query('DELETE FROM {mm_block} WHERE bid = %d', $form_state['values']['bid']);
  db_query('DELETE FROM {mm_tree_block} WHERE bid = %d', $form_state['values']['bid']);
  drupal_set_message(t('The block has been deleted.'));
  if (drupal_get_destination()) return;
  $form_state['redirect'] = 'admin/build/block/mm';
}

/**
 * Display a list of links to popup tree browsers
 *
 * @return
 *   The HTML code for the links
 */
function mm_admin_browse() {
  mm_static('admin_browse', TRUE);
  $links = array();
  $list = array(
    t('Permission groups') => array(MM_BROWSER_MODE_ADMIN_GROUP, user_access('administer all groups'), mm_content_groups_mmtid()),
    t('Top-level menus') => array(MM_BROWSER_MODE_ADMIN_PAGE, user_access('administer all menus'), mm_home_mmtid()),
    t('User menus') => array(MM_BROWSER_MODE_ADMIN_USER, user_access('administer all users'), mm_content_users_mmtid()),
    t('Entire tree') => array(MM_BROWSER_MODE_ADMIN_PAGE, _mm_menu_access_all_admin(), 1),
  );

  foreach ($list as $text => $item) {
    if ($item[1]) {
      $links[] = array(
        'title' => $text,
        'href' => '#',
        'external' => TRUE,
        'attributes' => array(
          'onclick' => "return Drupal.MMAdminBrowse($item[2], '$item[0]', $item[2], '$text')",
        ),
      );
    }
  }

  return theme('links', $links, array());
}

/**
 * Print a CSV dump of the entire MM tree
 */
function mm_admin_dump() {
  class dumpIter {
    protected $fp;

    public function __construct() {
      $this->fp = fopen('php://output', 'w');
      fputcsv($this->fp, array(
          'visname', 'mmtid', 'level', 'name', 'alias', 'default_mode', 'owner', 'theme',
          'alw_theme', 'alw_type', 'hidden', 'groups_w', 'users_w', 'groups_a',
          'users_a', 'groups_u', 'users_u', 'groups_r', 'users_r', 'flags', 'block'));
    }

    public function iterate($item) {
      $visname = str_repeat('>', $item->level) . ' ' . $item->name;

      $allowed_themes = array();
      $allowed_node_types = array();
      $result = db_query("SELECT * FROM {mm_cascaded_settings} WHERE mmtid = %d AND type IN('allowed_themes', 'allowed_node_types')", $item->mmtid);
      while ($r = db_fetch_object($result)) {
        if ($r->type == 'allowed_themes') $allowed_themes[] = $r->data;
        else if ($r->type == 'allowed_node_types') $allowed_node_types[] = $r->data;
      }

      $groups = array(MM_PERMS_READ => array(), MM_PERMS_WRITE => array(), MM_PERMS_SUB => array(), MM_PERMS_APPLY => array());
      $users = array(MM_PERMS_READ => array(), MM_PERMS_WRITE => array(), MM_PERMS_SUB => array(), MM_PERMS_APPLY => array());
      $result =
        db_query('SELECT t2.mmtid,t2.name,a.mode FROM {mm_tree} t '.
                 'INNER JOIN {mm_tree_access} a ON a.mmtid=t.mmtid '.
                 'LEFT JOIN {mm_tree} t2 ON a.gid=t2.mmtid '.
                 'WHERE t2.mmtid>=0 AND a.mmtid=%d ORDER BY t2.name', $item->mmtid);
      while ($r = db_fetch_object($result))
        $groups[$r->mode][$r->mmtid] = $r->name;

      $result =
        db_query('SELECT a.mode,a.gid FROM {mm_tree} t INNER JOIN {mm_tree_access} a '.
                 'ON a.mmtid=t.mmtid '.
                 'WHERE a.gid<0 AND a.mmtid=%d', $item->mmtid);
      while ($r = db_fetch_object($result)) {
        $u = mm_content_get_users_in_group($r->gid, NULL, TRUE, 5);
        if (!is_null($u)) $users[$r->mode] = $u;
      }

      fputcsv($this->fp, array($visname, $item->mmtid, $item->level, $item->name,
          $item->alias, $item->default_mode, $item->uid, $item->theme,
          join(', ', $allowed_themes), join(', ', $allowed_node_types), $item->hidden,
          $this->dump($groups[MM_PERMS_WRITE]), $this->dump($users[MM_PERMS_WRITE]),
          $this->dump($groups[MM_PERMS_SUB]), $this->dump($users[MM_PERMS_SUB]),
          $this->dump($groups[MM_PERMS_APPLY]), $this->dump($users[MM_PERMS_APPLY]),
          $this->dump($groups[MM_PERMS_READ]), $this->dump($users[MM_PERMS_READ]),
          join(',', array_keys($item->flags)), $item->bid));

      return 1;
    }

    protected function dump($arr) {
      $out = array();
      foreach ($arr as $id => $name)
        if ($id) $out[] = "$name [$id]";
        else $out[] = $name;
      return join(', ', $out);
    }
  }

  header('Content-type: text/plain');
  $params = array(
    MM_GET_TREE_ITERATOR =>      new dumpIter(),
    MM_GET_TREE_RETURN_BLOCK =>  TRUE,
  );
  mm_content_get_tree(1, $params);
  exit();
}

function mm_admin_find_user(&$form_state) {
  $form['#theme'] = 'node_form';
  _mm_ui_userlist_setup(array(0 => ''), $form, 'userlist', t("User's name:"), TRUE, '');
  $form['userlist-choose']['#title'] = '';
  $form['buttons']['submit'] = array('#type' => 'submit',
      '#value' => t('View/edit user'));
  return $form;
}

function mm_admin_find_user_submit($form, &$form_state) {
  $uid = mm_ui_mmlist_key0($form_state['values']['userlist']);

  if (isset($uid)) {
    $form_state['redirect'] = array("user/$uid/edit", 'destination=admin/user/user/by-uid');
  }
}

/**
 * Check all mm_tree.sort_idx entries and optionally fix any that seem to be
 * incorrect.
 *
 * @param $fix
 *   If TRUE, queue any wrong entries to be fixed during monster_menus_exit().
 * @param $output_mode
 *   Sets the method for outputting any error messages. Currently, these methods
 *   are supported:
 *   - watchdog:  The Drupal watchdog() function (suitable for cron)
 *   - message:   The drupal_set_message() function, which sends output to the
 *                current web page (do not use in a batch script). This is the
 *                default method, and is used by the menu entry.
 *   - print:     Print the messages to standard i/o
 */
function mm_admin_validate_sort_index($fix = FALSE, $output_mode = 'message') {
  function _mm_admin_validate_sort_index_output($output_mode, $fix, $message, $strings) {
    static $errors = 0;

    if (is_null($message)) {
      if ($output_mode == 'message') {
        if ($errors) {
          if ($fix) {
            return format_plural($errors, '1 error was fixed.', '@count errors were fixed.');
          }
          return l(t('Click here to fix all errors'), 'admin/mm/sort/1');
        }
        return t('No errors found.');
      }
      return;
    }

    if ($errors++ == MM_ADMIN_VALIDATE_SORT_INDEX_MAX) {
      $message = 'Only the first !num messages are shown.';
      $strings = array('!num' => MM_ADMIN_VALIDATE_SORT_INDEX_MAX);
    }
    else if ($errors > MM_ADMIN_VALIDATE_SORT_INDEX_MAX) return;

    switch ($output_mode) {
      case 'watchdog':
        watchdog('mm', $message, $strings, WATCHDOG_ERROR);
        break;

      case 'message':
        if (!$fix) {
          drupal_set_message(t($message, $strings), 'error');
        }
        break;

      case 'print':
        print t($message, $strings) . "\n";
        break;
    }
  }

  $parents = array();
  $skip_bad = '';
  $q = db_query("SELECT t.* FROM (SELECT %d AS mmtid UNION SELECT p.mmtid FROM {mm_tree_parents} p WHERE p.parent = %d) x INNER JOIN {mm_tree} t ON t.mmtid = x.mmtid ORDER BY sort_idx", 1, 1);
  while ($r = db_fetch_object($q)) {
    if ($r->sort_idx_dirty) {
      $skip_bad = $r->sort_idx;
      _mm_admin_validate_sort_index_output($output_mode, $fix, 'mmtid=!m (parent=!par) is marked as dirty. Updating now.', array('!m' => $r->mmtid, '!par' => $r->parent));
      // An update was previously missed, so definitely do it now
      mm_content_update_sort_queue($r->parent);
    }

    if ($r->mmtid > 1) {
      $is_bad = FALSE;
      if ($r->parent == $prev->mmtid || isset($sibling) && $r->parent == $sibling->mmtid) {
        // Going deeper in tree
        $parents[] = isset($parent) ? $parent : FALSE;
        $parent = $r->parent == $prev->mmtid ? $prev : $sibling;
        $sibling = NULL;
      }
      else {
        if (strncmp($skip_bad, $r->sort_idx, strlen($skip_bad))) $skip_bad = '';

        $old_sib = $sibling;
        $old_parents = $parents;
        $old_parent = $parent;
        // See if we are going back up the tree
        while (count($parents) && $parent->mmtid != $r->parent) {
          $sibling = $parent;
          $parent = array_pop($parents);
        }

        // The parent was never found
        if (!is_object($parent) || $parent->mmtid != $r->parent) {
          // If the item's parent exists, this means the sort index is off
          if (mm_content_get($r->parent)) {
            // The index is bad, but don't complain here; do it later on
            $sibling = $old_sib;
            $parents = $old_parents;
            $parent = $old_parent;
            $is_bad = TRUE;
          }
          else {
            _mm_admin_validate_sort_index_output($output_mode, FALSE, 'MM Tree entry !mmtid has missing parent !parent. Testing halted.', array('!mmtid' => $r->mmtid, '!parent' => $r->parent));
            return _mm_admin_validate_sort_index_output($output_mode, FALSE, NULL, NULL);
          }
        }
      }

      if (empty($skip_bad)) {
        if ($r->parent != $parent->mmtid) {
          _mm_admin_validate_sort_index_output($output_mode, $fix, 'Entry at mmtid=!m has a parent=!par that is inconsistent with its sort order.', array('!m' => $r->mmtid, '!par' => $r->parent));
          $is_bad = TRUE;
        }
        else if (strlen($r->sort_idx) - strlen($parent->sort_idx) != MM_CONTENT_BTOA_CHARS) {
          // The length of this entry's index is not correct, relative to its parent
          $msg = strlen($r->sort_idx) > strlen($parent->sort_idx) ? 'Sort index at mmtid=!m (parent=!par) is too long.' : 'Sort index at mmtid=!m (parent=!par) is too short.';
          _mm_admin_validate_sort_index_output($output_mode, $fix, $msg, array('!m' => $r->mmtid, '!par' => $r->parent));
          $is_bad = TRUE;
        }
        else {
          if ($r->sort_idx == $prev->sort_idx) {
            // Entry has same index as its predecessor
            _mm_admin_validate_sort_index_output($output_mode, $fix, 'mmtid=!m1 (parent=!par1) and mmtid=!m2 (parent=!par2) have the same sort index.', array('!m1' => $r->mmtid, '!par1' => $r->parent, '!m2' => $prev->mmtid, '!par2' => $prev->parent));
            $is_bad = TRUE;
          }
          else if (strncmp($r->sort_idx, $parent->sort_idx, strlen($parent->sort_idx))) {
            // The indices should match, up to the parent's length
            _mm_admin_validate_sort_index_output($output_mode, $fix, 'Sort indices of child and parent do not match at mmtid=!m (parent=!par).', array('!m' => $r->mmtid, '!par' => $r->parent));
            $is_bad = TRUE;
          }
          else if (isset($sibling)) {
            if (substr($r->sort_idx, -MM_CONTENT_BTOA_CHARS) == substr($sibling->sort_idx, -MM_CONTENT_BTOA_CHARS)) {
              // Entry has same index as its last sibling
              _mm_admin_validate_sort_index_output($output_mode, $fix, 'Siblings mmtid=!m1 and mmtid=!m2 have the same sort index (their parent=!par).', array('!m1' => $r->mmtid, '!m2' => $sibling->mmtid, '!par' => $r->parent));
              $is_bad = TRUE;
            }
            else {
              $msg = '';
              if ($sibling->name == MM_ENTRY_NAME_RECYCLE) {
                $msg = 'Entry mmtid=!m1 comes after mmtid=!m2, which is a recycle bin (their parent=!par).';
              }
              else if ($r->name != MM_ENTRY_NAME_RECYCLE && !$r->hidden) {
                if ($sibling->hidden) {
                  $msg = 'Entry mmtid=!m1 is not hidden, but it comes after mmtid=!m2 which is (their parent=!par).';
                }
                else if ($r->weight < $sibling->weight) {
                  $msg = 'Entry mmtid=!m1 has a weight that is lower than mmtid=!m2 (their parent=!par).';
                }
                else if ($r->weight == $sibling->weight && strcasecmp($r->name, $sibling->name) < 0) {
                  // The simple cases are tested above, but strcasecmp() is not
                  // the same as collation-based comparisons in the DB, so upon
                  // failure, ask the DB if the sort is correct
                  if (db_result(db_query('SELECT t1.name < t2.name FROM {mm_tree} t1 INNER JOIN {mm_tree} t2 ON t2.parent = t1.parent WHERE t1.mmtid = %d AND t2.mmtid = %d', $r->mmtid, $sibling->mmtid))) {
                    $msg = 'Entry mmtid=!m1 comes after mmtid=!m2, even though it has a name that is earlier alphabetically (their parent=!par).';
                  }
                }
              }

              if ($msg) {
                _mm_admin_validate_sort_index_output($output_mode, $fix, $msg, array('!m1' => $r->mmtid, '!m2' => $sibling->mmtid, '!par' => $r->parent));
                $is_bad = TRUE;
              }
            }
          }
        }
      }

      if ($is_bad) {
        if ($fix) mm_content_update_sort_queue($r->parent, NULL, TRUE);
        if (empty($skip_bad)) $skip_bad = $r->sort_idx;
      }
      $sibling = $r;
    }
    $prev = $r;
  }

  return _mm_admin_validate_sort_index_output($output_mode, $fix, NULL, NULL);
}

function mm_admin_reassign_content() {
  $form['old_user'] = array(
    '#type' => 'textfield',
    '#title' => t('Old username'),
  );
  $form['new_user'] = array(
    '#type' => 'textfield',
    '#title' => t('New username'),
  );
  $form['migrate_user_content'] = array(
    '#type' => 'checkbox',
    '#title' => t('Migrate content in personal user space'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}

function mm_admin_reassign_content_validate($form, &$form_state) {
  $old = user_load(array('name' => $form_state['values']['old_user']));
  $new = user_load(array('name' => $form_state['values']['new_user']));
  if (empty($old)) form_set_error('old_user', t('Could not find a match for old user.'));
  if (empty($new)) form_set_error('new_user', t('Could not find a match for new user.'));
  if ($form_state['values']["migrate_user_content"] == 1) {
    if (empty($old->user_mmtid)) {
      form_set_error('migrate_user_content', t('Unable to find the current user\'s home directory.  No further processing was done.'));
    }
  }
}

function mm_admin_reassign_content_submit($form, &$form_state) {
  $old = user_load(array('name' => $form_state['values']['old_user']));
  $new = user_load(array('name' => $form_state['values']['new_user']));
  $exclude_where = '';
  $args = array($old->uid);
  if (empty($form_state['values']['migrate_user_content']) && !empty($old->user_mmtid)) {
    $mmtids = array();
    $result = db_query('SELECT DISTINCT mmtid FROM {mm_tree_parents} WHERE parent = %d', $old->user_mmtid);
    $mmtids[] = $old->user_mmtid;
    while ($item = db_fetch_object($result)) {
      $mmtids[] = $item->mmtid;
    }
    $exclude_where = ' AND mmtid NOT IN (' . db_placeholders($mmtids) . ')';
    $args = array_merge($args, $mmtids);
  }

  $result = db_query('SELECT mmtid FROM {mm_tree} WHERE uid = %d' . $exclude_where, $args);
  $i = 0;
  while ($item = db_fetch_object($result)) {
    mm_content_update_quick(array('uid' => $new->uid), array('mmtid' => $item->mmtid));
    $i++;
  }
  drupal_set_message(t('Groups and pages that have switched owners: @count', array('@count' => $i)));

  array_unshift($args, $new->uid);
  db_query('UPDATE {node} n, {mm_node2tree} m SET n.uid = %d WHERE n.uid = %d AND n.nid = m.nid' . $exclude_where, $args);
  drupal_set_message(t('Nodes that have switched users: @count', array('@count' => db_affected_rows())));

  db_query('DELETE FROM {mm_group} WHERE uid = %d', $old->uid);
  drupal_set_message(t('Number of groups the user has been removed from: @count', array('@count' => db_affected_rows())));

  $vgroups = '';
  foreach (mm_content_get_uids_in_group(NULL, $old->uid, FALSE, TRUE, FALSE) as $gid) {
    $vgroups .= '<br />' . l(mm_content_get_name($gid), "mm/$gid");
  }
  if ($vgroups) {
    drupal_set_message(t('The user is still a member of these virtual group(s):') . $vgroups);
  }
}
