<?php

/**
 * @file
 * Admin forms for MM CCK Permissions
 */

define('MM_CCK_PERMS_VIEW_ID', MM_PERMS_READ);
define('MM_CCK_PERMS_EDIT_ID', MM_PERMS_WRITE);

function mm_cck_perms_convert($form_state) {
  drupal_set_message(t('Warning: The conversion process wipes out all current MM CCK Permissions and replaces them with the settings you choose below.'), 'warning');
  $form = array();

  $types = array(
    MM_CCK_PERMS_EDIT_ID => array(
      t('<em>edit *</em> Permission'),
      "If a field's edit permission is checked, it will get these MM CCK edit permissions.",
    ),
    MM_CCK_PERMS_VIEW_ID => array(
      t('<em>view *</em> Permission'),
      "If a field's view permission is checked, it will get these MM CCK view permissions.",
    ),
  );
  module_load_include('inc', 'monster_menus', 'mm_ui_content_edit');

  foreach (user_roles() as $rid => $name) {
    $form['role_names'][$rid] = array('#value' => $name);
    if ($rid == DRUPAL_ANONYMOUS_RID) {
      $form['permissions'][$rid] = array('#value' => t("If a view or edit permission is checked for the anonymous user, then all users will have that type of access to the field in MM CCK's permisisons."));
    }
    else {
      $form['permissions'][$rid] = array();
      $preselect = array();
      $is_auth_rid = FALSE;
      if ($rid == DRUPAL_AUTHENTICATED_RID && _mm_cck_perms_convert_find_all_users_group()) {
        $is_auth_rid = TRUE;
        $preselect = array(MM_CCK_PERMS_VIEW_ID, MM_CCK_PERMS_EDIT_ID);
      }
      mm_ui_content_edit_permissions($form['permissions'][$rid], $types, $preselect, array(), array(), NULL, FALSE, FALSE, array(), "-$rid");
      if ($is_auth_rid) {
        $form['permissions'][$rid]['table']['everyone']['#value']['title'] = '';
        $form['permissions'][$rid]['table']['everyone'][0][0]['#value'] = '<div class="mm-permissions-group-everyone-' . DRUPAL_AUTHENTICATED_RID . '">' . t('All logged-in users') . '</div>';
        unset($form['permissions'][$rid]['table']['indiv']);
        unset($form['permissions'][$rid]['table']['groups']);
      }
    }
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Convert'),
  );
  return $form;
}

function _mm_cck_perms_convert_find_all_users_group() {
  static $gid;

  if (!isset($gid)) {
    $gid = FALSE;
    $vgroup_start = mm_content_get(array('parent' => mm_content_groups_mmtid(), 'name' => MM_ENTRY_NAME_VIRTUAL_GROUP));
    if ($vgroup_start) {
      $group = mm_content_get(array('parent' => $vgroup_start[0]->mmtid, 'name' => 'All logged-in users'));
      if ($group) {
        $gid = $group[0]->mmtid;
      }
    }
  }
  return $gid;
}

function mm_cck_perms_convert_submit($form, $form_state) {
  $vals =& $form_state['values'];

  // Needed for content_field_instance_update()
  module_load_include('inc', 'content', 'includes/content.crud');
  $submitted_perms = array();
  foreach (array_keys(user_roles()) as $rid) {
    if (isset($vals["all_values_group-$rid"]) && isset($vals["all_values_user-$rid"])) {
      list($submitted_perms[$rid]['groups'], $submitted_perms[$rid]['users'], $submitted_perms[$rid]['default_modes']) = _mm_ui_form_parse_perms($vals, TRUE, "-$rid");
    }
  }
  if (_mm_cck_perms_convert_find_all_users_group()) {
    foreach ($submitted_perms[DRUPAL_AUTHENTICATED_RID]['default_modes'] as $mode) {
      $submitted_perms[DRUPAL_AUTHENTICATED_RID]['groups'][$mode] = array(_mm_cck_perms_convert_find_all_users_group());
    }
    $submitted_perms[DRUPAL_AUTHENTICATED_RID]['default_modes'] = array();
  }

  $types = content_types();
  foreach ($types as $type_name => $type) {
    $new_perms = array();
    $fields = $type['fields'];
    foreach ($fields as $field_name => $field) {
      $new_perms[$field_name] = _mm_cck_perms_convert_get_field_perms($submitted_perms, $field_name);
      _mm_cck_perms_convert_perms_compact($new_perms[$field_name], 'mm_cck_');
    }
    unset($new_default_perms);
    unset($last);
    $all_perms_same = TRUE;
    foreach ($new_perms as $p) {
      if (isset($last)) {
        if (!_mm_cck_perms_convert_perms_are_equal($p, $last)) {
          $all_perms_same = FALSE;
          break;
        }
      }
      $last = $p;
    }

    $use_defaults = array(
      'mm_cck_use_defaults' => TRUE,
      'mm_cck_default_modes' => array(),
      'mm_cck_users' => array(),
      'mm_cck_groups' => array(),
    );
    if ($new_perms && $all_perms_same) {
      foreach ($fields as $field_name => $field) {
        if (!isset($new_default_perms)) {
          $new_default_perms = array(
            'modes' => $new_perms[$field_name]['mm_cck_default_modes'],
            'users' => $new_perms[$field_name]['mm_cck_users'],
            'groups' => $new_perms[$field_name]['mm_cck_groups'],
          );
        }
        $fields[$field_name]['widget'] = $use_defaults + $fields[$field_name]['widget'];
      }
    }
    else {
      $new_default_perms = array(
        'modes' => array(MM_PERMS_WRITE),
        'users' => array(),
        'groups' => array(),
      );
      foreach ($fields as $field_name => $field) {
        if ($new_perms[$field_name]['mm_cck_default_modes'] == array(MM_PERMS_WRITE)) {
          $new_perms[$field_name] = $use_defaults;
        }
        $fields[$field_name]['widget'] = $new_perms[$field_name] + $fields[$field_name]['widget'];
      }
    }

    _mm_cck_perms_convert_perms_compact($new_default_perms, '');
    foreach ($new_default_perms as $perm_name => $value) {
      variable_set('mm_cck_default_' . $perm_name . '_' . $type_name, $value);
    }

    foreach ($fields as $field_name => $field) {
      content_field_instance_update($field, FALSE);
    }
  }

  content_clear_type_cache(TRUE);
  drupal_set_message(t('Process complete.'));
}

function _mm_cck_perms_convert_perms_compact(&$value, $prefix) {
  $all_users = _mm_cck_perms_convert_find_all_users_group();

  if ($all_users && isset($value[$prefix . 'groups'][MM_PERMS_WRITE]) && in_array($all_users, $value[$prefix . 'groups'][MM_PERMS_WRITE])) {
    $value[$prefix . 'groups'][MM_PERMS_WRITE] = array($all_users);
    unset($value[$prefix . 'groups'][MM_PERMS_READ]);
    unset($value[$prefix . 'users'][MM_PERMS_WRITE]);
    unset($value[$prefix . 'users'][MM_PERMS_READ]);
  }
  else if ($all_users && isset($value[$prefix . 'groups'][MM_PERMS_READ]) && in_array($all_users, $value[$prefix . 'groups'][MM_PERMS_READ])) {
    $value[$prefix . 'groups'][MM_PERMS_READ] = array($all_users);
    unset($value[$prefix . 'users'][MM_PERMS_READ]);
  }
  else {
    if (empty($value[$prefix . 'users'][MM_PERMS_WRITE])) {
      unset($value[$prefix . 'users'][MM_PERMS_WRITE]);
    }

    if (empty($value[$prefix . 'users'][MM_PERMS_READ])) {
      unset($value[$prefix . 'users'][MM_PERMS_READ]);
    }

    if (empty($value[$prefix . 'groups'][MM_PERMS_WRITE])) {
      unset($value[$prefix . 'groups'][MM_PERMS_WRITE]);
    }

    if (empty($value[$prefix . 'groups'][MM_PERMS_READ])) {
      unset($value[$prefix . 'groups'][MM_PERMS_READ]);
    }
  }
}

function _mm_cck_perms_convert_perms_are_equal($perm1, $perm2) {
  if ($perm1['mm_cck_use_defaults'] != $perm2['mm_cck_use_defaults'] ||
    $perm1['mm_cck_default_modes'] != $perm2['mm_cck_default_modes']) {
    return FALSE;
  }

  foreach (array('mm_cck_users', 'mm_cck_groups') as $type) {
    foreach (array(MM_PERMS_WRITE, MM_PERMS_READ) as $mode) {
      if (isset($perm1[$type][$mode]) != isset($perm2[$type][$mode])) {
        return FALSE;
      }
      if (isset($perm1[$type][$mode]) && $perm1[$type][$mode] != $perm2[$type][$mode]) {
        return FALSE;
      }
    }
  }

  return TRUE;
}

function _mm_cck_perms_convert_get_field_perms($submitted_perms, $field_name) {
  static $role_perms;

  if (!isset($role_perms)) {
    $role_perms = array();
    foreach (content_fields() as $field) {
      $write = array_keys(user_roles(FALSE, 'edit ' . $field['field_name']));
      $role_perms[$field['field_name']][MM_PERMS_WRITE] = $write;
      $role_perms[$field['field_name']][MM_PERMS_READ] = array_diff(array_keys(user_roles(FALSE, 'view ' . $field['field_name'])), $write);
    }
  }
  $out = array(
    'mm_cck_use_defaults' => 0,
    'mm_cck_default_modes' => array(),
    'mm_cck_users' => array(MM_PERMS_WRITE => array(), MM_PERMS_READ => array()),
    'mm_cck_groups' => array(MM_PERMS_WRITE => array(), MM_PERMS_READ => array()),
  );
  foreach (array(MM_PERMS_WRITE, MM_PERMS_READ) as $mode) {
    foreach ($role_perms[$field_name][$mode] as $rid) {
      if ($rid == DRUPAL_ANONYMOUS_RID || in_array($mode, $submitted_perms[$rid]['default_modes'])) {
        $out['mm_cck_default_modes'][$mode] = TRUE;
        $out['mm_cck_users'][$mode] = array();
        $out['mm_cck_groups'][$mode] = array();
        $everyone = TRUE;
        break;
      }
      else if (empty($out['mm_cck_default_modes'][$mode])) {
        if ($rid == DRUPAL_AUTHENTICATED_RID && !empty($submitted_perms[DRUPAL_AUTHENTICATED_RID]['groups'][$mode])) {
          $out['mm_cck_users'][$mode] = array();
          $out['mm_cck_groups'][$mode] = $submitted_perms[DRUPAL_AUTHENTICATED_RID]['groups'][$mode];
          break;
        }
        else {
          if (isset($submitted_perms[$rid]['users'][$mode])) {
            $out['mm_cck_users'][$mode] = array_merge($out['mm_cck_users'][$mode], $submitted_perms[$rid]['users'][$mode]);
          }
          if (isset($submitted_perms[$rid]['groups'][$mode])) {
            $out['mm_cck_groups'][$mode] = array_merge($out['mm_cck_groups'][$mode], $submitted_perms[$rid]['groups'][$mode]);
          }
          if ($mode == MM_PERMS_READ) {
            if (isset($submitted_perms[$rid]['users'][MM_PERMS_WRITE])) {
              $out['mm_cck_users'][$mode] = array_merge($out['mm_cck_users'][$mode], $submitted_perms[$rid]['users'][MM_PERMS_WRITE]);
            }
            if (isset($submitted_perms[$rid]['groups'][MM_PERMS_WRITE])) {
              $out['mm_cck_groups'][$mode] = array_merge($out['mm_cck_groups'][$mode], $submitted_perms[$rid]['groups'][MM_PERMS_WRITE]);
            }
          }
        }
      }
    }
    if (!empty($everyone)) break;
  }
  $out['mm_cck_default_modes'] = array_keys($out['mm_cck_default_modes']);
  $out['mm_cck_users'][MM_PERMS_WRITE] = array_unique($out['mm_cck_users'][MM_PERMS_WRITE]);
  $out['mm_cck_users'][MM_PERMS_READ] = array_unique($out['mm_cck_users'][MM_PERMS_READ]);
  $out['mm_cck_groups'][MM_PERMS_WRITE] = array_unique($out['mm_cck_groups'][MM_PERMS_WRITE]);
  $out['mm_cck_groups'][MM_PERMS_READ] = array_unique($out['mm_cck_groups'][MM_PERMS_READ]);
  return $out;
}

function theme_mm_cck_perms_convert($form) {
  mm_static('settings_perms', TRUE, 'mm-cck-perms-convert', 'edit-submit');

  $rows = array();
  foreach (element_children($form['role_names']) as $key) {
    if (is_array($form['permissions'][$key])) {
      $row = array();
      $row[] = array('data' => drupal_render($form['role_names'][$key]), 'class' => 'permission');
      $row[] = array('data' => drupal_render($form['permissions'][$key]));
      $rows[] = $row;
    }
  }
  $header = array(t('Role'), t('Permissions'));
  $output = theme('table', $header, $rows, array('id' => 'mm-cck-perms-convert-table'));
  $output .= drupal_render($form);
  return $output;
}
