/*
 * Ext JS Library 2.0 RC 1
 * Copyright(c) 2006-2007, Ext JS, LLC.
 * licensing@extjs.com
 * 
 * http://extjs.com/license
 */

FeedPanel = function() {
  FeedPanel.superclass.constructor.call(this, {
    id:            'feed-tree',
    region:        'west',
    title:         'Feeds',
    split:         true,
    width:         225,
    height:        400,
    minSize:       175,
    collapsible:   true,
    margins:       '5 0 10 5',
    cmargins:      '5 5 10 5',
    rootVisible:   false,
    lines:         false,
    autoScroll:    true,
    root:          new Ext.tree.TreeNode('Feed Viewer'),
    collapseFirst: false,
    tbar:          Drupal.settings.MMFV.allowAddDel ? [{
      iconCls: 'add-feed',
      text:    'Add Feed',
      handler: this.showWindow,
      scope:   this
    },{
      id:      'delete',
      iconCls: 'delete-icon',
      text:    'Remove',
      handler: function() {
        var s = this.getSelectionModel().getSelectedNode();
        if(s) this.removeFeed(s);
      },
      scope:   this
    }] : null
  });

  this.feeds = [];

  this.getSelectionModel().on({
    selectionchange: function(sm, node) {
      if(node && node.isLeaf()) {
        this.fireEvent('feedselect', node.attributes);
      }
      if(Drupal.settings.MMFV.allowAddDel) this.getTopToolbar().items.get('delete').setDisabled(!node || node.attributes.readonly);
    },

    scope:           this
  });

  this.addEvents({feedselect: true});

  this.on('contextmenu', this.onContextMenu, this);
};

Ext.extend(FeedPanel, Ext.tree.TreePanel, {
  onContextMenu: function(node, e) {
    var items;
    if(node) {
      items = [{
        id:      'load',
        iconCls: 'load-icon',
        text:    node.isSelected() ? 'Refresh Feed' : 'Load Feed',
        scope:   this,
        handler: function() {
          if( this.ctxNode.isSelected() ) this.ctxNode.unselect();
          this.ctxNode.select();
        }
      }];
      if(Drupal.settings.MMFV.allowAddDel) {
        items.push({
          id:      'remove',
          text:    'Remove',
          iconCls: 'delete-icon',
          scope:   this,
          handler: function() {
            this.ctxNode.ui.removeClass('x-node-ctx');
            this.removeFeed(this.ctxNode);
            this.ctxNode = null;
          }
        });
        items.push({
          id:      'add-feed',
          iconCls: 'add-feed',
          text:    'Add Feed',
          handler: this.showWindow,
          scope:   this
        });
      }
    }
    else if(Drupal.settings.MMFV.allowAddDel) {
      items = [{
        id:      'add-feed',
        iconCls: 'add-feed',
        text:    'Add Feed',
        handler: this.showWindow,
        scope:   this
      }];
    }
    else return false;

    this.menu = new Ext.menu.Menu({
      id:    'feeds-ctx',
      items: items
    });
    this.menu.on('hide', this.onContextHide, this);

    if(this.ctxNode) {
      this.ctxNode.ui.removeClass('x-node-ctx');
      this.ctxNode = null;
    }
    this.ctxNode = node;
    if(node) {
      this.ctxNode.ui.addClass('x-node-ctx');
      this.menu.items.get('load').setDisabled(!node.isLeaf());
      if(Drupal.settings.MMFV.allowAddDel) {
        this.menu.items.get('remove').setDisabled(node.attributes.readonly);
        this.menu.items.get('add-feed').setDisabled(node.attributes.readonly);
      }
    }
    this.menu.showAt(e.getXY());
  },

  onContextHide: function() {
    if(this.ctxNode) {
      this.ctxNode.ui.removeClass('x-node-ctx');
      this.ctxNode = null;
    }
  },

  showWindow: function() {
    if(!this.win) {
      this.win = new FeedWindow();
      this.win.on('validfeed', this.addCatAndFeed, this);
    }
    var node, str;
    if(this.ctxNode) node = this.ctxNode;
    else node = this.getSelectionModel().getSelectedNode();

    if(node && node.isLeaf()) node = node.parentNode;
    if(node && !node.attributes.readonly) str = node.text;
    this.win.show(str, this);
  },

  removeFeed: function(node) {
    if( !node.isLeaf() && node.childNodes.length && !node.attributes.delOK ) {
      this.deleteNode = node;
      return Ext.Msg.confirm('Remove category', String.format('Are you sure you want to permanently remove the category "{0}" and the {1} feed(s) it contains?', node.text, node.childNodes.length),
          function(butn) {
            if(butn=='yes') {
              this.deleteNode.attributes.delOK = true;
              this.removeFeed(this.deleteNode)
            }
          },
        this);
    }
    var url = Drupal.settings.MMFV['delete']+'/';
    url += node.isLeaf() ? node.parentNode.attributes.key+'/'+node.attributes.key : node.attributes.key;
    Ext.Ajax.request({url: url});
    node.unselect();
    node.collapse();
    Ext.fly(node.ui.elNode).slideOut('l', {
      callback: node.remove,
      scope:    node,
      duration: .4
    });
  },

  addCatAndFeed: function(catKey, catName, attrs) {
    var where = this.root.findChild('key', catKey);
    if(!where) this.addCategory(catKey, catName, true);
    else this.feeds = where;
    this.addFeed(attrs);
  },

  addCategory: function(key, name, expanded, readonly) {
    this.feeds = this.root.appendChild(
      new Ext.tree.TreeNode({
        key:      key,
        text:     name,
        cls:      'feeds-node',
        expanded: expanded,
        readonly: readonly,
        listeners: {
          expand: function(node) {
            FeedViewer.saveState('expand', node.attributes.key);
          },
          collapse: function(node) {
            FeedViewer.saveState('collapse', node.attributes.key);
          }
        }
      })
    );
  },

  testExistingFeed: function(data, type, catName) {
    var ret;
    this.root.eachChild( function(item) {
        if(item.attributes.text.toLowerCase() == catName.toLowerCase()) {
          ret = item.findChild('data', data);
          if( ret && ret.attributes.type != type ) ret = null;
          return false;
        }
      }
    );
    return ret;
  },

  addFeed: function(attrs, inactive, preventAnim) {
    Ext.apply(attrs, {
      iconCls:     'feed-icon',
      leaf:        true,
      cls:         'feed',
      getState:    function() {
        return { selected: this.isSelected() };
      },
      stateEvents: ['click', 'select']
    });
    var node = new Ext.tree.TreeNode(attrs);
    var where, first;
    if(!preventAnim) {
      this.feeds.eachChild( function(item) {
          if(node.text.toLowerCase() < item.text.toLowerCase()) {
            if(!where) first = item;
            return false;
          }
          where = item;
        }
      );
    }
    if(first) this.feeds.insertBefore(node, first);
    else if(where && where.nextSibling) this.feeds.insertBefore(node, where.nextSibling);
    else this.feeds.appendChild(node);
    if(!inactive) {
      if(!preventAnim) {
        Ext.fly(node.ui.elNode).slideIn('l', {
          callback: node.select,
          scope:    node,
          duration: .4
        });
      }
      else {
        node.select();
      }
    }
    return node;
  },

  // prevent the default context menu when you miss the node
  afterRender: function() {
    FeedPanel.superclass.afterRender.call(this);
    this.el.on('contextmenu', function(e) {
      e.preventDefault();
      if( e.target.className == 'x-panel-body' || e.target.className == 'x-tree-node' || e.target.className == 'x-tree-node-ct' ) this.onContextMenu(null, e);
    }, this);
  }
});
