/*
 * Ext JS Library 2.0 RC 1
 * Copyright(c) 2006-2007, Ext JS, LLC.
 * licensing@extjs.com
 * 
 * http://extjs.com/license
 */

MainPanel = function() {
  this.preview = new Ext.Panel({
    id:         'preview',
    region:     'south',
    cls:        'preview',
    autoScroll: true,

    tbar: [{
      id:      'win',
      text:    'Full Article',
      iconCls: 'new-win',
      disabled: true,
      scope:    this,
      handler:  function() {
        window.location = this.gsm.getSelected().data.link;
      }
    }]
  });

  this.grid = new FeedGrid(this, {
    tbar:[{
        split:   true,
        text:    'Reading Pane',
        tooltip: {title: 'Reading Pane',
                  text:  'Show, move or hide the Reading Pane'},
        iconCls: 'preview-bottom',
        handler: this.movePreview.createDelegate(this, []),
        menu:    {
          id:    'reading-menu',
          cls:   'reading-menu',
          width: 100,
          items: [{
            id:           'prev-bottom',
            text:         'Bottom',
            checked:      true,
            group:        'rp-group',
            checkHandler: this.movePreview,
            scope:        this,
            iconCls:      'preview-bottom'
          },{
            id:           'prev-right',
            text:         'Right',
            checked:      false,
            group:        'rp-group',
            checkHandler: this.movePreview,
            scope:        this,
            iconCls:      'preview-right'
          },{
            id:           'prev-hide',
            text:         'Hide',
            checked:      false,
            group:        'rp-group',
            checkHandler: this.movePreview,
            scope:        this,
            iconCls:      'preview-hide'
          }]
        }
      },
      {
        id:            'summary',
        pressed:       Drupal.settings.MMFV.lastState.summary,
        enableToggle:  true,
        text:          'Summary',
        tooltip:       {title: 'Post Summary',
                        text:  'View a short summary of each item in the list'},
        iconCls:       'summary',
        scope:         this,
        toggleHandler: function(btn, pressed) {
          this.grid.togglePreview(pressed);
          FeedViewer.saveState('summary', pressed);
        }
      },
      '-',
      {
        id:            'source',
        text:          'Source',
        tooltip:       {title: 'Feed Source',
                        text:  'View the original Web page for this feed'},
        iconCls:       'new-win',
        disabled:      true,
        scope:         this,
        handler:       function() {
          window.location = this.feedLink;
        }
      },
      {
        id:            'button0',
        text:          ' ',
        hidden:        true,
        iconCls:       'new-win',
        scope:         this,
        handler:       function() {
          window.location = this.buttonLink[0];
        }
      },
      {
        id:            'button1',
        text:          ' ',
        hidden:        true,
        iconCls:       'new-win',
        scope:         this,
        handler:       function() {
          window.location = this.buttonLink[1];
        }
      },
      {
        id:            'button2',
        text:          ' ',
        hidden:        true,
        iconCls:       'new-win',
        scope:         this,
        handler:       function() {
          window.location = this.buttonLink[2];
        }
      }
    ],
    listeners: {
      render:   function() {
        var mode = Drupal.settings.MMFV.lastState.preview || 'prev-bottom';
        Ext.getCmp(mode).setChecked(true);
      },
      scope:    this
    }
  });

  MainPanel.superclass.constructor.call(this, {
    region:   'center',
    margins:  '5 5 10 0',
    border:   false,
    id:       'main-view',
    layout:   'border',
    title:    'Loading...',
    hideMode: 'offsets',
    items:[
      this.grid, {
        id:     'bottom-preview',
        layout: 'fit',
        items:  this.preview,
        height: 200,
        split:  true,
        border: false,
        region: 'south'
      }, {
        id:     'right-preview',
        layout: 'fit',
        border: false,
        region: 'east',
        width:  350,
        split:  true,
        hidden: true
      }]
  });

  this.gsm = this.grid.getSelectionModel();

  this.gsm.on('rowselect', function(sm, index, record) {
    FeedViewer.getTemplate().overwrite(this.preview.body, record.data);
    this.preview.topToolbar.items.get('win').enable();
  }, this, {buffer: 250});

  this.grid.store.on('beforeload', function() {
    this.preview.body.update('');
    this.preview.topToolbar.items.get('win').disable();
    this.feedLink = '';
    Ext.getCmp('source').disable();
  }, this);
  this.grid.store.on('load', function() {
    this.feedLink = Ext.DomQuery.selectValue('link', this.grid.store.reader.xmlData.getElementsByTagName('channel'));
    Ext.getCmp('source').setDisabled(!this.feedLink);
    this.gsm.selectFirstRow();
  }, this);
};

Ext.extend(MainPanel, Ext.Panel, {
  loadFeed: function(feed) {
    this.grid.loadFeed(feed.key, feed.isTid);
    Ext.getCmp('main-view').setTitle(feed.text);
    this.buttonLink = [];
    for(var i=0; i<3; i++) {
      var b = Ext.getCmp('button'+i);
      if(feed.button && feed.button[i]) {
        b.setText(feed.button[i].text);
// this doesn't work        b.tooltip = feed.button[i].tip.text;
        b.show();
        this.buttonLink[i] = feed.button[i].url;
      }
      else b.hide();
    }
  },

  movePreview: function(m, pressed) {
    if(!m) { // cycle if not a menu item click
      var readMenu = Ext.menu.MenuMgr.get('reading-menu');
      readMenu.render();
      var items = readMenu.items.items;
      var b = items[0], r = items[1], h = items[2];
      if(b.checked) {
        r.setChecked(true);
      }
      else if(r.checked) {
        h.setChecked(true);
      }
      else if(h.checked) {
        b.setChecked(true);
      }
      return;
    }
    if(pressed) {
      var preview = this.preview;
      var right = Ext.getCmp('right-preview');
      var bot = Ext.getCmp('bottom-preview');
      var btn = this.grid.getTopToolbar().items.get(0);
      FeedViewer.saveState('preview', m.id);
      switch(m.id) {
        case 'prev-bottom':
          right.hide();
          bot.add(preview);
          bot.show();
          bot.ownerCt.doLayout();
          btn.setIconClass('preview-bottom');
          this.grid.un('rowdblclick', this.openWindow, this);
          break;
        case 'prev-right':
          bot.hide();
          right.add(preview);
          right.show();
          right.ownerCt.doLayout();
          btn.setIconClass('preview-right');
          this.grid.un('rowdblclick', this.openWindow, this);
          break;
        case 'prev-hide':
          preview.ownerCt.hide();
          preview.ownerCt.ownerCt.doLayout();
          btn.setIconClass('preview-hide');
          this.grid.on('rowdblclick', this.openWindow, this);
          break;
      }
    }
  },

  openWindow: function(record) {
    record = (record && record.data) ? record : this.gsm.getSelected();
    window.location = record.data.link;
  }
});
